"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _CRLF, _CRLF_BYTES, _CRLF_BYTES_LENGTH, _DASHES, _encoder, _footer, _form;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Encoder = void 0;
const createBoundary_1 = __importDefault(require("./util/createBoundary"));
const isFormData_1 = __importDefault(require("./util/isFormData"));
const isFile_1 = __importDefault(require("./util/isFile"));
class Encoder {
    /**
     * Creates a multipart/form-data encoder.
     *
     * @param form - FormData object to encode. This object must be a spec-compatible FormData implementation.
     * @param boundary - An optional boundary string that will be used by the encoder. If there's no boundary string is present, Encoder will generate it automatically.
     *
     * @example
     *
     * import {Encoder} from "form-data-encoder"
     * import {FormData} from "formdata-node"
     *
     * const fd = new FormData()
     *
     * fd.set("greeting", "Hello, World!")
     *
     * const encoder = new Encoder(fd)
     */
    constructor(form, boundary = createBoundary_1.default()) {
        _CRLF.set(this, void 0);
        _CRLF_BYTES.set(this, void 0);
        _CRLF_BYTES_LENGTH.set(this, void 0);
        _DASHES.set(this, "-".repeat(2));
        _encoder.set(this, void 0);
        /**
         * Returns field's footer
         */
        _footer.set(this, void 0);
        /**
         * FormData instance
         */
        _form.set(this, void 0);
        if (!isFormData_1.default(form)) {
            throw new TypeError("Expected first argument to be a FormData instance.");
        }
        if (typeof boundary !== "string") {
            throw new TypeError("Expected boundary to be a string.");
        }
        this.boundary = boundary;
        this.contentType = `multipart/form-data; boundary=${this.boundary}`;
        __classPrivateFieldSet(this, _encoder, new TextEncoder());
        __classPrivateFieldSet(this, _CRLF, "\r\n");
        __classPrivateFieldSet(this, _CRLF_BYTES, __classPrivateFieldGet(this, _encoder).encode(__classPrivateFieldGet(this, _CRLF)));
        __classPrivateFieldSet(this, _CRLF_BYTES_LENGTH, __classPrivateFieldGet(this, _CRLF_BYTES).byteLength);
        __classPrivateFieldSet(this, _form, form);
        __classPrivateFieldSet(this, _footer, __classPrivateFieldGet(this, _encoder).encode(`${__classPrivateFieldGet(this, _DASHES)}${this.boundary}${__classPrivateFieldGet(this, _DASHES)}${__classPrivateFieldGet(this, _CRLF).repeat(2)}`));
        this.headers = Object.freeze({
            "Content-Type": this.contentType,
            "Content-Length": this.getContentLength()
        });
    }
    _getFieldHeader(name, value) {
        let header = "";
        header += `${__classPrivateFieldGet(this, _DASHES)}${this.boundary}${__classPrivateFieldGet(this, _CRLF)}`;
        header += `Content-Disposition: form-data; name="${name}"`;
        if (isFile_1.default(value)) {
            header += `; filename="${value.name}"${__classPrivateFieldGet(this, _CRLF)}`;
            header += `Content-Type: ${value.type || "application/octet-stream"}`;
        }
        return __classPrivateFieldGet(this, _encoder).encode(`${header}${__classPrivateFieldGet(this, _CRLF).repeat(2)}`);
    }
    /**
     * Returns form-data content length
     */
    getContentLength() {
        let length = 0;
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            length += this._getFieldHeader(name, value).byteLength;
            length += isFile_1.default(value)
                ? value.size
                : __classPrivateFieldGet(this, _encoder).encode(String(value)).byteLength;
            length += __classPrivateFieldGet(this, _CRLF_BYTES_LENGTH);
        }
        return length + __classPrivateFieldGet(this, _footer).byteLength;
    }
    /**
     * Creates an async iterator allowing to perform the encoding by portions.
     *
     * @example
     *
     * import {Readable} from "stream"
     *
     * import {FormData, File, fileFromPath} from "formdata-node"
     * import {Encoder} from "form-data-encoder"
     *
     * import fetch from "node-fetch"
     *
     * const fd = new FormData()
     *
     * fd.set("field", "Just a random string")
     * fd.set("file", new File(["Using files is class amazing"]))
     * fd.set("fileFromPath", await fileFromPath("path/to/a/file.txt"))
     *
     * const encoder = new Encoder(fd)
     *
     * const options = {
     *   method: "post",
     *   headers: encoder.headers,
     *   body: Readable.from(encoder.encode()) // or Readable.from(encoder)
     * }
     *
     * const response = await fetch("https://httpbin.org/post", options)
     *
     * console.log(await response.json())
     */
    async *encode() {
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            yield this._getFieldHeader(name, value);
            if (isFile_1.default(value)) {
                yield* value.stream();
            }
            else {
                yield __classPrivateFieldGet(this, _encoder).encode(String(value));
            }
            yield __classPrivateFieldGet(this, _CRLF_BYTES);
        }
        yield __classPrivateFieldGet(this, _footer);
    }
    [(_CRLF = new WeakMap(), _CRLF_BYTES = new WeakMap(), _CRLF_BYTES_LENGTH = new WeakMap(), _DASHES = new WeakMap(), _encoder = new WeakMap(), _footer = new WeakMap(), _form = new WeakMap(), Symbol.asyncIterator)]() {
        return this.encode();
    }
}
exports.Encoder = Encoder;
